#!/bin/bash

[ $0 = "./setup_qmrtg.sh" ] || { echo "Please run from the qmrtg package root directory" ; exit 1; } 

echo ====================================
echo
echo "This will setup MRTG to use QMRTG."
echo "If you want to stop, press Ctrl-C now. Otherwise ENTER."
read
echo

echo "Let's begin then."
echo "The full config is 8 steps long."
echo "I need to know something to adapt qmrtg to your"
echo "system. Please double-check any answer before confirming it."
echo


echo "1. mrtg config file path." 
echo "where do you hold MRTG stuff? I also expect to find cfg/mrtg.cfg here."
echo "Please enter the full path (directory)."
read MRTG_DOCUMENTROOT
[ -d $MRTG_DOCUMENTROOT ] || { echo "This directory does not exist. Please fix and re-run `basename $0`"; exit 1; }
[ -f $MRTG_DOCUMENTROOT/cfg/mrtg.cfg ] || { echo "I can not find \"$MRTG_DOCUMENTROOT/cfg/mrtg.cfg\". Re-run `basename $0` and fix."; exit 1; } 

echo

echo "2. database files root directory"
echo "qmrtg need to store a few very small files to keep analysis control stuff."
echo "these files must be in a safe place. They must not be dropped on reboots."
echo "BSD systems would put them under /var/db. Linux systems under /var/lib."
echo "A directory \"qmrtg/\" will be created below the path you specify here."
read QMRTG_DBROOTPATH
[ -d $QMRTG_DBROOTPATH ] || { echo "This directory does not exist. Please create it and re-run `basename $0`"; exit 1; }

echo

echo "3. qmrtg analysis"
echo "Do you want qmrtg to carry out every analysis it is able to do or do you"
echo "want to choose yourself what to analyze? Please type 'a' or 'b'."
read QMRTG_ANALYZERS
[ $QMRTG_ANALYZERS = "b" ] && {
	echo "Ok. You need to go manually then. Just copy any block you're interested"
	echo "in from the mrtg.cfg sample configuration file you find in the"
	echo "qmrtg package to the system mrtg.cfg file"
	exit 0
}
[ -z $QMRTG_ANALYZERS ] && { echo "Please answer the questions. Re-run `basename $0`"; exit 1; }

echo

echo "4. qmail settings: qmail-send"
echo "Where do you keep qmail-send log files? I expect them to be in multilog style."
read QMAIL_QMAILSEND
[ -d $QMAIL_QMAILSEND ] || { echo "I can not find this directory. Please re-run `basename $0` and fix"; exit 1; }

echo

echo "5. qmail settings: qmail-smtpd"
echo "Where do you keep qmail-smtpd log files? I expect them to be in multilog style."
read QMAIL_QMAILSMTPD
[ -d $QMAIL_QMAILSMTPD ] || { echo "I can't find this directory. Please re-run `basename $0` and fix"; exit 1; }

echo

echo "6. qmail settings: qmail-pop3d"
echo "Where do you keep qmail-pop3d log files? I expect them to be in multilog style."
read QMAIL_QMAILPOP3D
[ -d $QMAIL_QMAILPOP3D ] || { echo "I can't find this directory. Please re-run `basename $0` and fix"; exit 1; }

echo

echo "7. qmail settings: qmail queue"
echo "Where is your qmail queue? It's usually in /var/qmail/queue"
read QMAIL_QMAILQUEUE
[ -d $QMAIL_QMAILQUEUE ] || { echo "I can't find this directory. Please re-run `basename $0` and fix"; exit 1; }

echo
echo

echo "8. realizing configuration"
echo "This is what you chose:"
echo "mrtg documentroot: $MRTG_DOCUMENTROOT"
echo "database root directory: $QMRTG_DBROOTPATH"
echo "and you want to use every analyzer provided by qmrtg"
echo "qmail-send logfiles in: $QMAIL_QMAILSEND"
echo "qmail-smtpd logfiles in: $QMAIL_QMAILSMTPD"
echo "qmail-pop3d logfiles in: $QMAIL_QMAILPOP3D"

echo
echo "Is this ok? press ENTER to configure MRTG and QMRTG with those settings or"
echo "press Control-C to stop without doing anything."
read

echo

echo "configuring for QMRTG:"

cd ../examples

echo "	creating the database files home: $QMRTG_DBROOTPATH"
mkdir -p $QMRTG_DBROOTPATH/qmrtg || exit 1

echo

echo "	setting QMRTG to use $QMRTG_DBROOTPATH as db home"
sed "s/QMRTG_DBPATH=\/var\/db\/qmrtg/QMRTG_DBPATH=$QMRTG_DBROOTPATH\/qmrtg/g" < qmrtg.conf > /usr/local/etc/qmrtg.conf || exit 1

echo "	setting QMRTG for qmail logfiles paths"
cp mrtg.cfg mrtg.cfg_
sed "s/\/var\/log\/supervise\/qmail\/send/$QMAIL_QMAILSEND/g" < mrtg.cfg_ > mrtg.cfg__ && mv mrtg.cfg__ mrtg.cfg_
sed "s/\/var\/log\/supervise\/qmail\/smtpd/$QMAIL_QMAILSMTPD/g" < mrtg.cfg_ > mrtg.cfg__ && mv mrtg.cfg__ mrtg.cfg_
sed "s/\/var\/log\/supervise\/qmail\/pop3d/$QMAIL_QMAILPOP3D/g" < mrtg.cfg_ > mrtg.cfg__ && mv mrtg.cfg__ mrtg.cfg_
sed "s/\/var\/qmail\/queue/$QMAIL_QMAILQUEUE/g" < mrtg.cfg_ > mrtg.cfg__ && mv mrtg.cfg__ mrtg.cfg_


echo
echo

echo "configuring for MRTG"

echo "	updating mrtg.cfg to use qmrtg"
echo
tail -n$(( `wc -l mrtg.cfg_ | awk '{ print $1 }'` - 42 )) mrtg.cfg_ >> $MRTG_DOCUMENTROOT/cfg/mrtg.cfg || exit 1

echo
echo "Cleaning up..."
rm mrtg.cfg_

echo
echo "Everything done. Try manually executing \"mrtg $MRTG_DOCUMENTROOT/cfg/mrtg.cfg\" to see if things are ok."