/*
 *  qmrtg-concurrency.c
 *  qmrtg
 *
 *  Created by Mij <mij@bitchx.it> on Thu Feb 19 2004.
 *  Please have a look at the LICENCE file for informations
 *  regarding how to use, modify or redistribute this code.
 *
 */

#include "qmrtg-concurrency.h"
#include "qmrtgutils.h"

#define MAX(a,b)        (((a)>(b))?(a):(b))


#include <stdio.h>
/* strstr() */
#include <string.h>
/* for malloc() free() etc */
#include <stdlib.h>
/* for getopt() */
#include <unistd.h>
/* for round() */
#include <math.h>

/* print help */
void usage(void);

int main(int argc, char *argv[]) {
    FILE *file;
    char *from_timestamp;                   /* where to start parsing from */
    char *tempstr;
    char *logfilename;
    char buf[MAX_LOGLINE_LENGTH];           /* service stuff */
    char *buf2;
    char *last_valid_logentry;              /* to avoid errors on malformed logfiles */
    int inst_concurrency;
    int max_concurrency = 0;
    float avg_concurrency = 0.0;
    int i = 0;
    
    int behavior = BEHAVE_NO_MRTG;
    int ch;                                 /* for getopt() */

#ifdef DEBUG
    int entries_considered_num = 0;
#endif
    
    
    /* correct prog call? */
    while ((ch = getopt(argc, argv, "m")) != -1) {
        switch (ch) {
            case 'm':
                behavior = BEHAVE_MRTG_SHORT;
                break;
            default:
                usage();
        }
    }
    
    
    if ( argc < 3 ) {
        printf("Invalid call.\n");
        usage();
        exit(ERR_INVALID_USAGE);
    }
    
    from_timestamp = argv[optind];      /* next argument after option args */
    logfilename = argv[optind+1];       /* second next argument after option args */
    
    /* does the file exist? */
    if ( (file = fopen(logfilename, "r")) == NULL ) {
        perror("Error opening the log file");
        return ERR_LOGFILE_NOT_EXIST;
    }
    
    
    /* the first loop of the next cycle frees() last_valid_entry when it's
        * not been malloced. This skips runtime error messages */
    last_valid_logentry = NULL;
    
    while ( fgets(buf, MAX_LOGLINE_LENGTH, file) != NULL ) {
         /* is this log entry valid / sane */
        if ( (tempstr = getTaiStr(buf)) == NULL ) continue;
         /* if we're here then the last_valid_logentry has to
        be updated to the new log entry which has been proven valid */
        free(last_valid_logentry);
        last_valid_logentry = tempstr;
        
        /* did we already analyze did entry? */
        if ( !isAfter(tempstr, from_timestamp) ) continue;
       
        /* increments the total number of bytes with the value in this entry */
        if ( (buf2 = strstr(buf, " tcpserver: status: ")) != NULL ) {
            i++;
            inst_concurrency = strtol(buf2+20, (char **)NULL, 10);
            max_concurrency = ( inst_concurrency > max_concurrency ? inst_concurrency : max_concurrency );
            avg_concurrency = ( (avg_concurrency * (i-1)) + inst_concurrency ) / i;
#ifdef DEBUG
            entries_considered_num++;
#endif
        }
    }

    /* if we didn't terminate to the end of file, an error happened and buf can't be trusted
        * using !feof(file) would be more elegant/safe, but it is possible some new stuff appear
        * in the file between the failing fgets call and the if there */
/*
    if ( ferror(file) )
        snprintf(buf, MAX_LOGLINE_LENGTH, "@%s foobar", from_timestamp);
*/
    
    fclose(file);
    
    /*
     * fix "last_valid_logentry" if the file was empty
     * or not containing any valid multilog line
     */
    if (last_valid_logentry == NULL) {
        last_valid_logentry=(char *)malloc(TAI_STRING_LENGTH+1);
        strncpy(last_valid_logentry, from_timestamp, TAI_STRING_LENGTH);
    }
    
    
    tempstr = (char *)malloc(10); 
    sprintf(tempstr, "%f", rint(avg_concurrency) );
    i = 0;
     /* find and truncate the string (float) to an integer */
    while (tempstr[i++] != '.' && tempstr[i] != '\0') continue;
    tempstr[i-1] = '\0';

    /* these are the results */
    printf("%d\n%s\n", max_concurrency, tempstr);
    free(tempstr);
#ifdef DEBUG
    printf("bytes log entries: %d\n", entries_considered_num);
#endif
    
    
    switch (behavior) {
        case BEHAVE_NO_MRTG:
            /* print the timestamp of the last logfile line */
            printf("%s\n", last_valid_logentry);
            break;
        case BEHAVE_MRTG_SHORT:
            /* nothing more to do then */
            break;
        default: break;
    }
    
    
    
    return ERR_NO_ERRORS;
}


void usage(void) {
    printf("Usage:\nqmrt-concurrency opts timestamp logfile\nFor analyzing entries after timestamp in logfile.\n");
}

