/*
 *  qmrtg-messages.c
 *  qmrtg
 *
 *  Created by Mij <mij@bitchx.it> on Fri Feb 06 2004.
 *  Please have a look at the LICENCE file for informations
 *  regarding how to use, modify or redistribute this code.
 *
 */


#include "qmrtg-deliveries.h"
#include "qmrtgutils.h"

#define MAX(a,b)        (((a)>(b))?(a):(b))

#include <stdio.h>
 /* for strstr() */
#include <string.h>
 /* for malloc(), free(), exit() etc */
#include <stdlib.h>
 /* for getopt() */
#include <unistd.h>

 /* print help */
void usage(void);
 /* print the hostname */
void printHostname(void);


int main(int argc, char *argv[]) {
    FILE *file;
    char *from_timestamp;               /* the timestamp after when analyze. input by user */
    char *logfilename;                  /* the filename of the log to parse */
    char *tempstr;                      /* used here and there to point temporary stuff */
    char buf[MAX_LOGLINE_LENGTH];       /* used to store log entries one by one */
    char *last_valid_logentry;          /* to avoid problems with broken logfiles */

    long remote_deliveries = 0, local_deliveries = 0;
                                        /* keep track of the number of remote and local deliveries */
    int ch;                             /* for getopt() */
    int behavior = BEHAVE_NO_MRTG;

#ifdef DEBUG
    int entries_considered_num = 0;
#endif

    
    
     /* correct prog call? */
    while ((ch = getopt(argc, argv, "m")) != -1) {
        switch (ch) {
            case 'm':
                behavior = BEHAVE_MRTG_SHORT;
                break;
            default:
                usage();
        }
    }
    
    
    if ( argc < 3 ) {
        printf("Invalid call.\n");
        usage();
        exit(ERR_INVALID_USAGE);
    }
    
    from_timestamp = argv[optind];      /* next argument after option args */
    logfilename = argv[optind+1];       /* second next argument after option args */
    
#ifdef DEBUG
    printf("timestamp: %s\nfilename: %s\n", from_timestamp, logfilename);
#endif
    
    
     /* does the file exist? */
    if ( (file = fopen(logfilename, "r")) == NULL ) {
        perror("Error opening the log file");
        exit(ERR_LOGFILE_NOT_EXIST);
    }
    
    
    /* the first loop of the next cycle frees() last_valid_entry when it's
        * not been malloced. This skips runtime error messages */
    last_valid_logentry = NULL;
    
    while ( fgets(buf, MAX_LOGLINE_LENGTH, file) != NULL ) {
         /* is this log entry valid / sane */
        if ( (tempstr = getTaiStr(buf)) == NULL ) continue;
        
         /* if we're here then the last_valid_logentry has to
        be updated to the new log entry which has been proven valid */
        free(last_valid_logentry);
        last_valid_logentry = tempstr;
        
         /* did we already analyze did entry? */
        if ( !isAfter(tempstr, from_timestamp) ) continue;
        
         /* increment remote_deliveries if another remote delivery is found */
        if ( (tempstr = strstr(buf, " to remote ")) != NULL ) {
            remote_deliveries++;
#ifdef DEBUG
            entries_considered_num++;
#endif
        }
        
         /* increment locas_deliveries if another locals delivery is found */
        if ( (tempstr = strstr(buf, " to local ")) != NULL ) {
            local_deliveries++;
#ifdef DEBUG
            entries_considered_num++;
#endif
        }
    }
        
    /* if we didn't terminate to the end of file, an error happened and buf can't be trusted
     * using !feof(file) would be more elegant/safe, but it is possible some new stuff appear
     * in the file between the failing fgets call and the if there */
/*
    if ( ferror(file) )
        snprintf(buf, MAX_LOGLINE_LENGTH, "@%s foobar", from_timestamp);
*/
    
    fclose(file);

    /*
     * fix "last_valid_logentry" if the file was empty
     * or not containing any valid multilog line
     */
    if (last_valid_logentry == NULL) {
        last_valid_logentry=(char *)malloc(TAI_STRING_LENGTH+1);
        strncpy(last_valid_logentry, from_timestamp, TAI_STRING_LENGTH);
    }
    
    
     /* these are the results */
    printf("%ld\n%ld\n", remote_deliveries, local_deliveries);
#ifdef DEBUG
    printf("messages log entries: %d\n", entries_considered_num);
#endif
    
    switch (behavior) {
        case BEHAVE_NO_MRTG:
             /* print the timestamp of the last logfile line */
            printf("%s\n", last_valid_logentry);
            break;
        case BEHAVE_MRTG_SHORT:
             /* nothing more to do then */
            break;
        default: break;
    }
    
    exit(ERR_NO_ERRORS);
}


void usage(void) {
    printf("Usage:\nqmrt-messages opts timestamp logfile\nFor analyzing entries after timestamp in logfile.\n");
}

