/*
 *  qmrtg-queue.h
 *  qmrtg
 *
 *  Created by Mij <mij@bitchx.it> on Wed Jan 21 2004.
 *  Please have a look at the LICENCE file for informations
 *  regarding how to use, modify or redistribute this code.
 *
 */

#include "qmrtgutils.h"

#define MAX(a,b)        (((a)>(b))?(a):(b))

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <sys/stat.h>
#include <dirent.h>


int count_files_in(char *queuedir);
void usage();

int main(int argc, char *argv[]) {
    int err;
    char tempstr[MAX_PATH_LENGTH];       /* a temporary buffer */
    char queuedir[MAX_PATH_LENGTH];      /* the path of the qmail queue (argv[1]) */
    
     /* some safe check */
    if ( argc != 2 ) {
        usage();
        return 1;
    }

    if (strlen(argv[1]) > MAX_PATH_LENGTH-1) {
        printf("Maximum string size for the directory name exceeded.\nPlease recompile qmrtg-queue if you want it working with this huge path.\n");
        return 1;
    }    
    strncpy(queuedir, argv[1], MAX_PATH_LENGTH);
    
    
     /* now, @ work */

     /* for $QUEUE/mess : already preprocessed msgs */
    snprintf(tempstr, MAX_PATH_LENGTH-strlen(queuedir)-5, "%s/mess", queuedir);
    if ((err = count_files_in(tempstr)) < 0) {
        perror("Error opening QUEUE/mess");
#ifdef DEBUG
        printf("Line # %d\n", __LINE__);
#endif
        return 1;
    }
    printf("%d\n", err);


     /* for $QUEUE/todo : still to be preprocessed msgs */
    snprintf(tempstr, MAX_PATH_LENGTH-strlen(queuedir)-5, "%s/todo", queuedir);
    if ((err = count_files_in(tempstr)) < 0) {
        perror("Error opening QUEUE/todo");
#ifdef DEBUG
        printf("Line # %d\n", __LINE__);
#endif
        return 1;
    }
    printf("%d\n", err);
    
    return 0;
}


 /* recursively counts the number of files under directory "queuedir" */
int count_files_in(char *queuedir) {
    DIR *dir;
    struct dirent *de;
    char newdir[MAX_PATH_LENGTH];
    int err;
    int count_sub = 0;          /* local files */
    int count_loc = 0;          /* files in subdirectories */

    if ( (dir = opendir(queuedir)) == NULL ) return -1;

     /* queue tree traversal (depth-first) */
    while ( (de = readdir(dir)) != NULL ) {
        if ( !strcmp(de->d_name, ".") || !strcmp(de->d_name, "..") ) continue;
        if ( de->d_type == DT_DIR ) {
            snprintf(newdir, MAX_PATH_LENGTH-strlen(queuedir), "%s/%s", queuedir, de->d_name);
            if ( (err = count_files_in(newdir)) < 0) {
                closedir(dir);
                return -1;
            } else
                count_sub += err;
        } else {
            count_loc++;
        }
    }

    closedir(dir);
    
    return (count_sub + count_loc);
}


void usage() {
    printf("qmrtg-queue: feeds mrtg with informations about the queue of a qmail mailserver.\n");
    printf("Usage:\n\tqmrtg-queue <qmail_queue_dir>\n");
    printf("where qmail_queue_dir is the absolute path of the qmail queue directory.\n");
}

