/*
 *  qmrtgutils.c
 *  qmrtg
 *
 *  Created by Mij <mij@bitchx.it> on Thu Feb 05 2004.
 *  Please have a look at the LICENCE file for informations
 *  regarding how to use, modify or redistribute this code.
 *
 */

#include "qmrtgutils.h"


#include <stdio.h>
 /* for strcmp() */
#include <string.h>
 /* for malloc() and free() etc */
#include <stdlib.h>
 /* for gethostname() */
#include <unistd.h>
 /* for isdigit() */
#include <ctype.h>
 /* for strlen() */
#include <string.h>

#ifdef __FreeBSD__
 /* for sysctl() */
#include <sys/types.h>
#include <sys/sysctl.h>
 /* for struct timeval */
#include <time.h>
 /* for gettimeofday() */
#include <sys/time.h>
#endif

 /* 
  * returns a new string containing the timestamp
  * extracted from "string". "string" is expected to
  * be a multilog-style entry ("@timestamp logmsg")
  */
char *getTaiStr(char *string) {
    char *taistring = (char *)malloc(TAI_STRING_LENGTH+1);
    
    /* safety checks */
    if ( !isTai(&string[1]) ) {
        return NULL;
    }
    
    bcopy(string + 1, taistring, TAI_STRING_LENGTH);
    taistring[24] = '\0';
    
    return taistring;
}

 /*
  * chronologically compares the two timestamps
  * taken as argument
  */
Bool isAfter(char *what, char *than) {
    if ( what == NULL || than == NULL ) return false;
    return ( strcmp(what, than) > 0 );
}


/* 
 * true if candidate_str is
 * _sintactically_ a valid Tai string.
 * no checks are made on its actual content/meaning
 */
Bool isTai(char *candidate_str) {
    int i;

    if (candidate_str == NULL) return false;
    if (strlen(candidate_str) < TAI_STRING_LENGTH) return false;
    
    for (i=0; i < TAI_STRING_LENGTH; i++)
        if ( !isalnum(candidate_str[i]) ) return false;
    
    return true;
}


/**
 * compiles *hostnamestr with the machine's hostname
 */
void getHostname(char *hostnamestr) {
    if (hostnamestr == NULL) return;
    
    if ( gethostname(hostnamestr, MAX_HOSTNAME_LENGTH) != 0 ) {
        perror("Unable to get the hostname");
        exit(ERR_HOSTNAME);
    }
}

