/*
 *  qmrtgutils.h
 *  qmrtg
 *
 *  Created by Mij <mij@bitchx.it> on Thu Feb 05 2004.
 *  Please have a look at the LICENCE file for informations
 *  regarding how to use, modify or redistribute this code.
 *
 */


 /* meters and conventions */
#define TAI_STRING_LENGTH           24
#define MAX_LOGLINE_LENGTH          500
#define MAX_HOSTNAME_LENGTH         200
#define MAX_UPTIMESTR_LENGTH        20
#define MAX_PATH_LENGTH             80

#ifdef __FreeBSD__
 /* htf does Linux call this? */
#define SYSCTL_BOOTTIME_MIB_NAME   KERN_BOOTTIME
#endif

 /* behavior styles */
 /* print results and the last log entry analyzed */
#define BEHAVE_NO_MRTG              0
 /* only print results */
#define BEHAVE_MRTG_SHORT           1
 /* print results, then the uptime, then the name of the host */
#define BEHAVE_MRTG_LONG            2

 /* errors */
#define ERR_NO_ERRORS               0
#define ERR_INVALID_USAGE           1
#define ERR_LOGFILE_NOT_EXIST       5
#define ERR_HOSTNAME                6
#define ERR_SYSCTL                  7
#define ERR_GETCURRTIME             8

typedef enum { false, true } Bool;


/* 
 * returns a new string containing the timestamp
 * extracted from "string". "string" is expected to
 * be a multilog-style entry ("@timestamp log_message")
 */
char *getTaiStr(char *string);


/*
 * chronologically compares the two timestamps
 * taken as argument
 */
Bool isAfter(char *what, char *than);

/* 
 * true if candidate_str is
 * _sintactically_ a valid Tai string.
 * no checks are made on its actual content/meaning
 */
Bool isTai(char *candidate_str);

/* compiles hostnamestr with this machine's hostname*/
void getHostname(char *hostnamestr);
